using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

/*!
 * \mainpage LibMC.NET
 * 
 * \author Douglas P. Stark<br>UC Davis and Sandia National Labs<br>dpstark\@sandia.gov
 * 
 * \section intro_sec Introduction
 * Welcome to the LibMC.NET documentation. LibMC.NET is .NET class library
 * that wraps Mobile-C in a layer to make it accessible to .NET applications.
 * While the wrapper is continually under development, most of the features
 * of Mobile-C are currently available to the programmer. 
 * \htmlonly 
 * Please use the links above to find out more.
 * \endhtmlonly
 */

/**
 * \page installation Installing LibMC.NET
 * Installing LibMC.NET is straighforward but involves several steps. 
 * 
 * \section requirements Requirements
 * In order to use LibMC.NET you will need the following:
 * <ul>
 * <li> Ch version 6.0.0 or greater from http://www.softintegration.com/.
 * </li>
 * <li> Embedded Ch version 6.0.0 or greater, also from http://www.softintegration.com/.
 * </li>
 * <li> Mobile-C 1.10.0 or greater. See Section \ref downloading for instructions 
 * on how to obtain Mobile-C.
 * </li>
 * <li> Visual Studio 2005 or later. Express versions of Visual Studio can be found at 
 * http://www.microsoft.com/express/.
 * </li>
 * </ul>
 * 
 * \section downloading Downloading Mobile-C
 * First, you must obtain a version of the Mobile-C source code. If you are reading
 * this, chances are you have already completed this step. If you have not already 
 * downloaded the source code, it can be done in one of three ways:
 * 
 * <ul>
 * <li>Download a supported release of Mobile-C. Visit 
 * <a href ="http://www.mobilec.org/download.php">the Mobile-C website</a> for more
 * information on supported releases.</li>
 * <li>Download the latest source (unsupported) from 
 * <a href = "http://sourceforge.net/projects/mobilec/">Sourceforge.</a> This will
 * give you the most current version of Mobile-C, but not necessarily the most
 * stable version.</li>
 * <li>Check out the latest source code from the SVN repository. This requires 
 * that you have a subversion client installed. More information can be found at
 * <a href = "http://sourceforge.net/svn/?group_id=185466">this location.</a></li>
 * </ul>
 * 
 * \section build_mc Building the Mobile-C Libraries
 * Once you have obtained the Mobile-C source, please see the Mobile-C User's Guide
 * for information on compiling Mobile-C under Windows. Currently, only the Visual 
 * Studio .NET 2005 project is supported for LibMC.NET. Section 2.3 of the User's 
 * Guide describes how to compile Mobile-C into a static library. For LibMC.NET,
 * at least one of two configurations are required: the "Debug_DLL" or "Release_DLL"
 * versions. To build either one, select the appropriate configuration (this replaces
 * step 3 in the User's Guide, Section 2.3.1) and build the solution (step 4). 
 * Alternatively, you may select "Batch Build" from the "Build" menu and build all
 * four possible configurations.
 * 
 * \section install_mc Install the Mobile-C Libraries
 * After building the Mobile-C DLL files, the project will automatically copy the 
 * files to the system directory. By default, the files are copied to C:/Windows/System32/.
 * If your system is configured differently or you wish to change the installation 
 * directory, right-click on the mc_lib_win32 project in the Solution Explorer and
 * select "Properties." In the mc_lib_win32 Property Pages treeview, select 
 * "Configuration Properties," then "Build Events," and finally "Post-Build Event."
 * You can then change the "Command Line" field to copy the files to the directory 
 * of your choice. If you change the installation directory, be sure that your chosen
 * directory is in the system path and that you remove any other versions of the files.
 * You will also need to execute a "Rebuild" on the project to ensure the files are 
 * copied to the new location.
 * 
 * \section build_net Build LibMC.NET
 * Once you have built the Mobile-C DLL files, you can build LibMC.NET. Open the LibMC.NET
 * solution file located in the directory you installed or checked out Mobile-C to at 
 * src/win32/LibMC.NET/LibMC.Net.sln. From the "Build" menu, select "Rebuild Solution." 
 * You may want to build both the "Debug" and "Release" versions, or perform a batch build 
 * as described previously.
 */

/**
 * \page started Getting Started
 * LibMC.NET is very easy to use. The demo programs provided with the download are a good 
 * place to start. Please see the Examples section for more information.
 * 
 * \section build_demo Build the Demo Programs
 * The LibMC.NET demo program solution is located in the directory you installed or checked 
 * out Mobile-C to at demos/win32/LibMC.NET/LibMCDemos.sln. As before, select "Rebuild Solution"
 * from the "Build" menu. Note that the demo program solution contains the LibMC.NET project
 * as well. You may also build LibMC.NET from within the demo program solution.
 * 
 * By default, the LibMCGui demo is selected in the demo program solution. You may run this
 * program by selecting the "Debug" menu then "Start Debugging" or by pressing F5. Other
 * demo programs can be started by right-clicking the project in the Solution Explorer and 
 * selecting "Debug" then "Start new instance." The demo programs have their own documentation
 * as well. See the README file in src/win32/LibMC.NET or demos/win32/LibMC.NET for information
 * on how to build the demo program documentation.
 */

/**
 * \page using Using LibMC.NET
 * This section explains how to use the LibMC.NET class library in your .NET project. Currently,
 * it only describes the process for using the library in a C# console or GUI application. Other
 * languages, such as VB and managed C++, will require similar actions.
 * 
 * \section project Create a Project
 * First, create the type of project you would like to use from the Visual Studio "Start Page" or
 * the "File" menu. Select the name and location of the project as you would any other project. 
 * Second, add a reference to the configuration of LibMC.NET you would like to use. For debugging
 * purposes, the "Debug" configuration is probably best. To add the reference, right click the 
 * "References" item in the Solution Explorer for the project you just created. Select the "Browse"
 * tab and navigate to the output directory of the LibMC.NET project. The directory is located at
 * src/win32/LibMC.NET/bin/Configuration/ in the Mobile-C source directory, where Configuration
 * is either "Debug" or "Release." Select the DLL file and click "Ok." The References item in the 
 * Solution Explorer should now list "LibMC." Be sure to save the solution at this point.
 * 
 * \section code_use Using LibMC.NET Classes and Functions
 * As with any other namespace, you must add the declaration "using LibMC;" to any file you want to
 * have access to the class libraries. Once you have added the using statement, you can declare
 * objects from the library as you normally would declare any other objects. See the example programs
 * for more details.
 * 
 * \section code_misc Other Options
 * You may want to enable one or more features in your project that can help you use LibMC.NET or
 * debug problems. If you add any XML files to your project, you probably will want to set their 
 * properties in the project to copy the files to the output directory. This is done by selecting
 * the file in the Solution Explorer, opening its properties, and setting two fields:
 * 
 * <ul>
 * <li>Set the "Build Action" field to "Content" if it is not already set. This will make the file
 * part of the project should you decide to publish or package it.</li>
 * <li>Set the "Copy to output directory" to "Copy if newer" or "Copy always." This will copy the file
 * when you build the project.</li>
 * </ul>
 * 
 * There is also one important note regarding XML files in Visual Studio. <i>Do not create XML files
 * from within Visual Studio.</i> The Visual Studio XML file template contains a few leading characters
 * that specify the encoding of the file. They are hidden and you will not be able to change them.
 * These characters are not currently supported by Mobile-C and will crash a receiving agency.
 * 
 * To open the project properties, right-click the project in the Solution Explorer and select
 * "Properties." In the "Debug" pane, you may wish to set an alternate working directory for the 
 * project if you want easy access to XML files outside of the project. This is useful for debugging,
 * but may result in errors finding files if you package the project or create an installer. In
 * general, it is best to specify all files with full paths because the Mobile-C library loads from
 * a different location than the project. If you would like to be able to debug the Mobile-C library, 
 * you should select the "Enable unmanaged code debugging" check box. This will allow you to more 
 * easily see any errors that may occur in the unmanaged library, though hopefully none will.
 */

/**
 * \page common_tasks Common Operations
 * This section contains examples of commonly used operations for three main LibMC classes:
 * <ul>
 * <li>
 * \subpage mcagency The mobile agent agency.
 * </li>
 * <li>
 * \subpage mcagent Mobile agents.
 * </li>
 * <li>
 * \subpage mcaclmessage Agent communication language messages.
 * </li>
 * </ul>
 * 
 * For complete programs and more detailed examples, see the Examples section.
 */

/**
 * \page mcagency MCAgency
 * Examples of commonly used MCAgency operations:
 * 
 * Declare an agency as a member of a class:
 * \code
 * public static MCAgency Agency = new MCAgency();
 * \endcode
 * 
 * Set the agency's port:
 * \code
 * int temp = 5051;
 * Agency.Port = temp;
 * \endcode
 *
 * Start an agency:
 * \code
 * int temp;
 * temp = Agency.Initialize();
 * if (temp != 0)
 *     Console.WriteLine("Initialize: " + temp.ToString());
 * \endcode
 * 
 * Pause and resume an agency:
 * \code
 * Agency.HaltAgency();
 * Agency.ResumeAgency();
 * \endcode
 * 
 * Turn off the command prompt thread:
 * \code
 * temp = Agency.SetThreadOff(MCAgency.MC_ThreadIndex_e.MC_THREAD_CP);
 * if (temp != 0)
 *     Console.WriteLine("SetThreadOff: " + temp.ToString());
 * \endcode
 * 
 * Load an agent into a local agency:
 * \code
 * String filename = "agent.xml";
 * try
 * {
 *     Agency.LoadAgentMigrationMessageFile(filename);
 * }
 * catch (Exception ex)
 * {
 *     Console.WriteLine("Error loading file: " + ex.Message);
 * }
 * \endcode
 * Ideally, the file name should be specified absolutely. 
 * 
 * Send an agent to a remote agency:
 * \code
 * String filename = "agent.xml";
 * String ip = "192.168.23.93";
 * int port = 5051;
 * try
 * {
 *     Agency.SendAgentMigrationMessageFile(filename, ip, port);
 * }
 * catch (Exception ex)
 * {
 *     Console.WriteLine("Error sending file: " + ex.Message);
 * }
 * \endcode
 * 
 * Find an agent by name:
 * \code
 * MCAgent agent;
 * try
 * {
 *     agent = Agency.FindAgentByName("persistent1");
 * }
 * catch (Exception e)
 * {
 *     Console.WriteLine("Exception: " + e.Message);
 * }
 * \endcode
 *
 * Wait for an agent to arrive:
 * \code
 * MCAgent agent;
 * Agency.ResetSignal();
 * try
 * {
 *     agent = Agency.WaitRetrieveAgent();
 * }
 * catch (Exception e)
 * {
 *     Console.WriteLine("Exception: " + e.Message);
 * }
 * \endcode
 *
 * Wait indefinitely while an agency runs:
 * \code
 * Agency.MainLoop();
 * \endcode
 */

/** \file MCAgency.cs
 * Defines the MCAgency object and its member functions.
 */

/**
 * \example LibMCGui/Form1.cs
 * \include LibMCGui/Program.cs
 * Basic Mobile-C Windows Forms demo program
 */

/** 
 * \example LibMCConsole/Program.cs
 * Basic Mobile-C console demo program
 */

/** 
 * \example LibMCMiscTest/Program.cs
 * Demonstrates miscellaneous Mobile-C functions.
 */

/** 
 * \example LibMCInterop/Program.cs
 * Demonstrates interfacing with a .dl file and calling
 * agent functions.
 */

/** 
 * \example cs2ch/cs2chfunc.c
 * C library example that allows agents to call CLI
 * functions.
 */

/** 
 * \example LibMCCppEx/LibMCCppEx.cpp
 * \dontinclude LibMCCppEx/Form1.h
 * \until #pragma region Windows Form Designer generated code
 * \skip #pragma endregion
 * \until };
 * \line }
 * Demonstrates using LibMC.NET from a VC++ program.
 */

/** 
 * \example LibMCVbEx/Form1.vb
 * Demonstrates using LibMC.NET from a VB program.
 */

/**
 * \brief      Namespace for the .NET wrapper for Mobile-C.
 * 
 * LibMC encapsulates the Mobile-C DLL for windows in an .NET class
 * library. .NET programs can access the library to create agencies,
 * connect to agencies, interact with agents, etc.
 */
namespace LibMC
{
    /**
     * \brief   Wrapper class for MCAgency_t structure.
     * 
     * This class provides an interface to the Mobile-C agency. Member
     * functions for the class are generally overloaded versions of the 
     * respective functions in the Mobile-C library. The class maintains a 
     * pointer to the Mobile-C agency in unmanaged memory. The pointer
     * is not accessible by the user.
     */
    public partial class MCAgency
    {
        /**
         * \brief   Enum for describing the state of the agency.
         * 
         * This enum is used to determine whether or not certain actions
         * should be permitted, such as halting, resuming, and ending an agency
         */
        public enum MCAgencyState
        {
            NoState = -1,       /*!< Default, uninitialized state */
            Initialized = 0,    /*!< Agency initialized, but not started */
            Running,            /*!< Agency is running */
            Halted,             /*!< Agency has been stopped (can be resumed) */
            Ended,              /*!< Agency is stopped (destroyed) */
        }

        /**
         * \brief   Ch shell type.
         * 
         * Used to set the shell type for the Ch interpretter.
         */
        public enum ChShellType
        {
            CH_REGULARCH = 0,   /*!< Default, regular shell */
            CH_SAFECH = 1,      /*!< Safe shell */
        }

        private IntPtr agency_p = IntPtr.Zero;
        private MCAgencyOptions_t options;
        private int port = -1;
        private MCAgencyState state = MCAgencyState.NoState;

        /** 
         * \brief       Default constructor
         * 
         * The default constructor for the MCAgency class. It creates a new agency,
         * default options for the agency, and initializes the agency. It does not
         * start the agency.
         */
        public MCAgency()
        {
            options = new MCAgencyOptions_t();
            _MC_InitializeAgencyOptions(ref options);
            state = MCAgencyState.Initialized;
        }

        private IntPtr Agency
        {
            get
            {
                if (agency_p == IntPtr.Zero)
                    throw new InvalidAgentException();
                else
                    return agency_p;
            }
            set
            {
                if (value == IntPtr.Zero)
                    throw new InvalidAgentException("Attempting to assign null pointer to agency internal pointer!");
                else if (agency_p == IntPtr.Zero)
                    agency_p = value;
                else
                    throw new InvalidAgentException("Attempting to assign new agency to non-zero agency pointer!");
            }
        }

        /**
         * \brief       Accessor for the port number of the agency.
         * 
         * Allows the user to set the agency port or get the port number while it 
         * is running.
         * 
         * \note        The port must be set before the agency is started. Once the agency
         *              is started, the port cannot be changed.
         */
        public int Port
        {
            get
            {
                return port;
            }
            set
            {
                port = value;
            }
        }

        /**
         * \brief       Accessor for the agency state.
         * 
         * Allows the user to query the state of the agency.
         * 
         * \note        The state cannot be set by the user. It is
         *              controlled internally.
         */
        public MCAgencyState State
        {
            get
            {
                return state;
            }
        }

        /*
         * Agency functions
         */

        /**
         * \brief       Starts the agency.
         * 
         * Starts the agency and sets the agency state.
         * 
         * \returns     0 on success, -1 on failure.
         * 
         * \note        The agency port and any other options must be set
         *              before calling this function.
         */
        public int Initialize()
        {
            if (port == -1)
                return -1;
            else
            {
                Agency = MCAgency._MC_Initialize(port, ref options);
                state = MCAgencyState.Running;
                return 0;
            }
        }

        /**
         * \brief       Stops and destroys the agency.
         * 
         * Stops the agency and sets the agency state appropriately.
         * 
         * \returns     The return value of the underlying MC_End function.
         * 
         * \note        This call will fail if the underlying Mobile-C agency
         *              is not in the correct state.
         */
        public int End()
        {
            int temp = MCAgency._MC_End(Agency);
            state = MCAgencyState.Ended;
            return temp;
        }

        /**
         * \brief       Initializes Ch options for the agency.
         * 
         * Can be used to set the home directory and shell mode for the Ch interpretter.
         * 
         * \param       shellType The type of shell Ch should use: CH_REGULARCH or CH_SAFECH.
         * \param       home The home directory Ch should use.
         * \returns     The return value of the underlying MC_ChInitializeOptions function.
         * 
         * \note        This function must be called before the agency is started.
         */
        public int ChInitializeOptions(ChShellType shellType, String home)
        {
            ChOptions_t options = new ChOptions_t();
            options.chhome = home;
            options.shelltype = (int)shellType;
            return _MC_ChInitializeOptions(Agency, options);
        }

        /* \brief       Sets all threads for the agency to "on."
         * 
         * Sets all threads for the agency to "on." This is also the default state
         * for the agency.
         * 
         * \returns     The return value of the underlying MC_SetThreadsAllOn function.
         * 
         * \note        This function must be called before the agency is started.
         */
        public int SetThreadsAllOn()
        {
            return _MC_SetThreadsAllOn(ref options);
        }

        /**
         * \brief       Sets all threads for the agency to "off."
         * 
         * Sets all threads for the agency to "off." Not recommended for use.
         * 
         * \returns     The return value of the underlying MC_SetThreadsAllOff function.
         * 
         * \note        This function must be called before the agency is started.
         */
        public int SetThreadsAllOff()
        {
            return _MC_SetThreadsAllOff(ref options);
        }

        /**
         * \brief       Sets an individual thread for the agency to "on."
         * 
         * Threads are on by default. If they have been turned off, this function
         * turns them on again.
         * 
         * \param       index The enum that identifies the thread to be turned on.
         * \returns     The return value of the underlying MC_SetThreadOn function.
         * 
         * \note        This function must be called before the agency is started.
         */
        public int SetThreadOn(MC_ThreadIndex_e index)
        {
            return _MC_SetThreadOn(ref options, index);
        }

        /**
         * \brief       Sets an individual thread for the agency to "off."
         * 
         * Most commonly used to turn the command prompt thread off.
         * 
         * \param       index The enum that identifies the thread to be turned off.
         * \returns     The return value of the underlying MC_SetThreadOff function.
         * 
         * \note        This function must be called before the agency is started.
         */
        public int SetThreadOff(MC_ThreadIndex_e index)
        {
            return _MC_SetThreadOff(ref options, index);
        }

        /**
         * \brief       Temporarily halts the agency.
         * 
         * Halts the agency until it is resumed or ended.
         * 
         * \returns     The return value of the underlying MC_HaltAgency function.
         * 
         * \note        The underlying Mobile-C agency must be in the correct state
         *              to call this function or it will fail.
         */
        public int HaltAgency()
        {
            int temp = _MC_HaltAgency(Agency);
            state = MCAgencyState.Halted;
            return temp;
        }

        /**
         * \brief       Resumes a halted agency.
         * 
         * Resumes a halted agency. Cannot be used on ended agencies.
         * 
         * \returns     The return value of the underlying MC_ResumeAgency function.
         * 
         * \note        The underlying Mobile-C agency must be in the correct state
         *              to call this function or it will fail.
         */
        public int ResumeAgency()
        {
            int temp = _MC_ResumeAgency(Agency);
            state = MCAgencyState.Running;
            return temp;
        }

        /**
         * \brief       Sets the default state of an agent in the agency.
         * 
         * Can be used to set the default status of agents, but most agents
         * managed their state on their own.
         * 
         * \param       status The enum that identifies the desired agent state.
         * \returns     The return value of the underlying MC_SetDefaultAgentStatus function.
         */
        public int SetDefaultAgentStatus(MCAgent.MC_AgentStatus_e status)
        {
            return _MC_SetDefaultAgentStatus(Agency, status);
        }

        /**
         * \brief       Waits for an agent to arrive and returns the agent.
         * 
         * Waits for an agent to arrive in the agency, then returns that agent.
         * The agent is not allowed to execute.
         * 
         * \returns     The agent that was retrieved or an empty agent if it fails.
         */
        public MCAgent WaitRetrieveAgent()
        {
            IntPtr temp = _MC_WaitRetrieveAgent(Agency);
            if (temp != IntPtr.Zero)
                return new MCAgent(temp);
            else
                return new MCAgent();
        }

        /**
         * \brief       Waits for an agent to arrive.
         * 
         * Waits for an agent to arrive in the agency. The agent is 
         * allowed to execute normally.
         * 
         * \returns     The return value of the underlying MC_WaitAgent function.
         */
        public int WaitAgent()
        {
            return _MC_WaitAgent(Agency);
        }

        /*
         * Migration functions
         */

        /**
         * \brief       Sends an agent migration message file to an agency.
         * 
         * Sends the specified XML file to another agency (local or remote).
         * 
         * \param       filename The name of the file to send (fully qualified).
         * \param       hostname The URL, IP address, or other identifier for the agency host.
         * \param       port The port to send to.
         * \returns     The return value of the underlying MC_SendAgentMigrationMessageFile function.
         */
        public int SendAgentMigrationMessageFile(String filename, String hostname, int port)
        {
            return MCAgency._MC_SendAgentMigrationMessageFile(Agency, filename, hostname, port);
        }

        /**
         * \brief       Load an agent migration message.
         * 
         * Loads the specified XML file to this agency automatically. There is 
         * no need to specify a port or agency location.
         * 
         * \param       filename The name of the file to send (fully qualified).
         * \returns     The return value of the underlying MC_SendAgentMigrationMessageFile function.
         */
        public int LoadAgentMigrationMessageFile(String filename)
        {
            return MCAgency._MC_SendAgentMigrationMessageFile(Agency, filename, "localhost", this.Port);
        }

        /**
         * \brief       Sends an agent migration message to an agency.
         * 
         * Sends an agent migration message to another agency (local or remote).
         * 
         * \param       message The agent migration message.
         * \param       hostname The URL, IP address, or other identifier for the agency host.
         * \param       port The port to send to.
         * \returns     The return value of the underlying MC_SendAgentMigrationMessageFile function.
         */
        public int SendAgentMigrationMessage(String message, String hostname, int port)
        {
            return _MC_SendAgentMigrationMessage(Agency, message, hostname, port);
        }

        /*
         * Signals and threading functions
         */

        /**
         * \brief       Broadcast a condition signal.
         * 
         * Broadcasts a signal in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit().
         * 
         * \param       id The ID number of the condition to signal.
         * \returns     The return value of the underlying MC_CondBroadcast function.
         */
        public int CondBroadcast(int id)
        {
            return _MC_CondBroadcast(Agency, id);
        }

        /**
         * \brief       Signal a condition.
         * 
         * Signals a condition in the agency. The parameter "id" is the ID of
         * the agency sync variable to signal that was created with SyncInit().
         * 
         * \param       id The ID number of the condition to signal.
         * \returns     The return value of the underlying MC_CondSignal function.
         */
        public int CondSignal(int id)
        {
            return _MC_CondSignal(Agency, id);
        }

        /**
         * \brief       Reset a condition signal.
         * 
         * Resets a signal in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit(). This function
         * must be called after a condition is received in order to clear it.
         * 
         * \param       id The ID number of the condition to reset.
         * \returns     The return value of the underlying MC_CondReset function.
         */
        public int CondReset(int id)
        {
            return _MC_CondReset(Agency, id);
        }

        /**
         * \brief       Wait for a condition signal.
         * 
         * Waits for a condition signal in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit(). This function blocks
         * until the signal is received.
         * 
         * \param       id The ID number of the condition to wait for.
         * \returns     The return value of the underlying MC_CondWait function.
         */
        public int CondWait(int id)
        {
            return _MC_CondWait(Agency, id);
        }

        /**
         * \brief       Lock a mutex.
         * 
         * Locks a mutex in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit(). This function
         * blocks until the mutex is locked.
         * 
         * \param       id The ID number of the mutex to lock.
         * \returns     The return value of the underlying MC_MutexLock function.
         */
        public int MutexLock(int id)
        {
            return _MC_MutexLock(Agency, id);
        }

        /**
         * \brief       Unlock a mutex.
         * 
         * Locks a mutex in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit().
         * 
         * \param       id The ID number of the mutex to unlock.
         * \returns     The return value of the underlying MC_MutexUnlock function.
         */
        public int MutexUnlock(int id)
        {
            return _MC_MutexUnlock(Agency, id);
        }

        /**
         * \brief       Posts a semaphore.
         * 
         * Posts a sempaphore in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit().
         * 
         * \param       id The ID number of the semaphore to post.
         * \returns     The return value of the underlying MC_SemaphorePost function.
         */
        public int SemaphorePost(int id)
        {
            return _MC_SemaphorePost(Agency, id);
        }

        /**
         * \brief       Wait for a semaphore to be posted.
         * 
         * Wait for a semaphore in the agency to be posted. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit(). This function blocks until
         * the semaphore is posted.
         * 
         * \param       id The ID number of the semaphore to wait for.
         * \returns     The return value of the underlying MC_SemaphoreWait function.
         */
        public int SemaphoreWait(int id)
        {
            return _MC_SemaphoreWait(Agency, id);
        }

        /**
         * \brief       Reset an agency signal.
         * 
         * Resets a signal in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit().
         * 
         * \returns     The return value of the underlying MC_ResetSignal function.
         */
        public int ResetSignal()
        {
            return _MC_ResetSignal(Agency);
        }

        /**
         * \brief       Delete a synchronization variable.
         * 
         * Deletes a synchronization variable in the agency. The parameter "id" is the ID of
         * the agency sync variable created with SyncInit().
         * 
         * \param       id The ID number of the variable to delete.
         * \returns     The return value of the underlying MC_SyncDelete function.
         */
        public int SyncDelete(int id)
        {
            return _MC_SyncDelete(Agency, id);
        }

        /**
         * \brief       Create a new synchronization variable.
         * 
         * Creates a new synchronization variable in the agency. The parameter "id" is 
         * desired ID of the variable. A random ID is returned if "id" is already in use.
         * 
         * \param       id The ID number of the condition to signal.
         * \returns     The return value of the underlying MC_CondBroadcast function-
         *              either a random ID or the desired ID if the desired ID is already
         *              in use.
         */
        public int SyncInit(int id)
        {
            return _MC_SyncInit(Agency, id);
        }

        /**
         * \brief       Wait for agency signals.
         * 
         * Waits for signals to occur in the agency.
         * 
         * \param       signals The ID number of the condition to signal.
         * \returns     The return value of the underlying MC_WaitSignal function.
         */
        public int WaitSignal(int signals)
        {
            return _MC_WaitSignal(Agency, signals);
        }

        /**
         * \brief       Delete a barrier object.
         * 
         * Deletes a barrier object from the agency. The parameter "id" is the ID of
         * the agency sync variable created with BarrierInit().
         * 
         * \param       id The ID number of the barrier to delete.
         * \returns     The return value of the underlying MC_BarrierDelete function.
         */
        public int BarrierDelete(int id)
        {
            return _MC_BarrierDelete(Agency, id);
        }

        /**
         * \brief       Create a new barrier.
         * 
         * Creates a new barrier object in the agency.
         * 
         * \param       id The ID number of the condition to signal.
         * \param       num_procs the number of process to block (?)
         * \returns     The return value of the underlying MC_BarrierInit function.
         */
        public int BarrierInit(int id, int num_procs)
        {
            return _MC_BarrierInit(Agency, id, num_procs);
        }

        /*
         * Steering functions
         */

        /**
         * \brief       Steering control function.
         * 
         * Really not sure.
         * 
         * \returns     The return value of the underlying MC_SteerControl function.
         * 
         * \todo Test MC_SteerControl, MC_Steer.
         */
        public MC_SteerCommand_e SteerControl()
        {
            return _MC_SteerControl();
        }

        /**
         * \brief       Steering control function.
         * 
         * Really not sure.
         * 
         * \param       funcptr Pointer to the steering function
         * \param       arg Argument to function
         * \returns     The return value of the underlying _MC_Steer function.
         * \note        This function does nothing but throw an exception right now.
         * \bug         MC_Steer is not yet implemented.
         * \todo        Implement MC_Steer
         */
        /*public int _MC_Steer(IntPtr agency, int (*funcptr)(void* data), void *arg);*/
        public int Steer(IntPtr funcptr, IntPtr arg)
        {
            //return _MC_Steer(Agency, IntPtr funcptr, IntPtr arg);
            throw new Exception("Steer() is not yet implemented!");
        }

        /*
         * Services
         */

        /**
         * \brief       Registers services in the agency.
         * 
         * Registers services provided by agents with the agency. Not really useful
         * in binary space.
         * 
         * \param       agent The agent providing the services.
         * \param       agentID The agent ID number.
         * \param       agentName The agent name.
         * \param       serviceNames An array of service names.
         * \param       numServices The number of services provided.
         * \returns     The return value of the underlying MC_RegisterService function.
         * 
         * \todo        Test MC_RegisterService and MC_SearchForService.
         */
        public int RegisterService(MCAgent agent, int agentID, String agentName, String[] serviceNames, int numServices)
        {
            return _MC_RegisterService(Agency, agent.Agent, agentID, agentName, serviceNames, numServices);
        }

        /**
         * \brief       Searches for services in the agency.
         * 
         * Searches for services provided by agents with the agency. Not really useful
         * in binary space.
         * 
         * \param       searchString The agent providing the services.
         * \param       agentNames The agent ID number.
         * \param       serviceNames The agent name.
         * \param       agentIDs An array of service names.
         * \param       numResults The number of services provided.
         * \returns     The return value of the underlying MC_SearchForService function.
         * 
         * \note        This function does nothing but throw an exception right now. 
         * \bug         MC_SearchForService is not yet implemented.
         * \todo        Implement SearchForService
         */
        /*private static extern int _MC_SearchForService(IntPtr agency, String searchString, char*** agentNames, char*** serviceNames, int** agentIDs, int* numResults);*/
        public int SearchForService(String searchString, IntPtr agentNames, IntPtr serviceNames, IntPtr agentIDs, IntPtr numResults)
        {
            //return _MC_SearchForService(IntPtr agency, String searchString, IntPtr agentNames, IntPtr serviceNames, IntPtr agentIDs, IntPtr numResults);
            throw new Exception("SearchForService() is not yet implemented!");
        }

        /*
         * Agent functions
        */

        /**
         * \brief       Add an agent to the agency.
         * 
         * Adds an agent to the agency.
         * 
         * \param       agent The agent to add.
         * \returns     The return value of the underlying MC_AddAgent function.
         */
        public int AddAgent(MCAgent agent)
        {
            return _MC_AddAgent(Agency, agent.Agent);
        }

        /**
         * \brief       Finds an agent by its name.
         * 
         * Finds an agent in the agency by its name.
         * 
         * \param       name The name of the agent to search for.
         * \returns     The return value of the underlying MC_FindAgentByName function.
         */
        public MCAgent FindAgentByName(String name)
        {
            return new MCAgent(_MC_FindAgentByName(Agency, name));
        }

        /**
         * \brief       Find an agent by its ID.
         * 
         * Finds an agent in the agency by its ID number.
         * 
         * \param       id The ID number of the condition to signal.
         * \returns     The return value of the underlying MC_FindAgentByID function.
         */
        public MCAgent FindAgentByID(int id)
        {
            return new MCAgent(_MC_FindAgentByID(Agency, id));
        }

        /**
         * \brief       Retrieve an agent from the agency.
         * 
         * Really not sure.
         * 
         * \returns     The return value of the underlying MC_CondBroadcast function.
         */
        public MCAgent RetrieveAgent()
        {
            return new MCAgent(_MC_RetrieveAgent(Agency));
        }

        /*
         * ACL Functions
         */

        /**
         * \brief       Send an ACL message to the agency.
         * 
         * Sends an ACL message to the agency. The message is delivered appropriately.
         * 
         * \param       acl_message The message to send.
         * \returns     The return value of the underlying MC_CondBroadcast function.
         */
        public int AclSend(MCAclMessage acl_message)
        {
            return MCAgency._MC_AclSend(Agency, acl_message.AclMsg);
        }

        /**
         * \brief       Makes the agency wait indefinitely.
         * 
         * Makes the agency wait indefinitely until it receives a "quit" command
         * or is otherwise terminated.
         * 
         * \returns     The return value of the underlying MC_MainLoop function.
         */
        public int MainLoop()
        {
            return _MC_MainLoop(Agency);
        }
    }

    /**
     * \brief   Exception class for use with null agency pointers
     * 
     * This class provides a way to inform the program that an
     * agency was created or accessed that had an invalid
     * internal pointer.
     */
    public class InvalidAgencyException : SystemException
    {
        private const String msg = "Private agency pointer not valid.";

        /**
         * \brief   Null agency pointer exception constructor.
         * 
         * Constructor for the exception class. This exception
         * simply defines a recognizable exception class and sets
         * the exception message appropriately.
         */
        public InvalidAgencyException()
            : base(msg)
        {
        }

        /**
         * \brief   Null agency pointer exception constructor.
         * 
         * Constructor for the exception class. Allows the use
         * of a custom error message.
         */
        public InvalidAgencyException(String exc)
            : base(exc)
        {
        }
    }
}
